# This file is a part of TkFax.
#

#
# Phone book database has the following structure:
# Each group has
#	- group name
#	- group description
#	- list of group members
# Each group member consists of
#	- fax number
#	- recipient
#	- reference
#	- description
#
# Phone book database is kept in two variables:
#
#   fax(grouplist)
#     = { {group0|description0} {group0|description0} ... }
#
#   fax(group-groupname)
#     = { {faxnumber0|recipient0|reference0|description0} ... }
#
# For delemeter(|), [TAB] will be used.
#
#---------------------------------------------------------------------

proc fax_phonebook {} {
#
# Phone Book Window
#
# 10 Feb 2005 Masaki Shinomiya:  merge pbook
# 19 Sep 2002 modified by R.Kawagishi
#
    global fax m pbookwindow2 pbook_sort_order pbook_grpsort_order
    global pbook_modified_flag wn

    # Name of this window

    set wn $fax(mwn).pbook
    set fax(phonebookwin) $wn
    if [winfo exists $wn] {return}
    toplevel $wn
    wm title $wn "TkFax: Phone Book"

    # Top label

    frame $wn.0   -bd 10
    label $wn.0.l -text "TkFax: Phone  Book" -padx 20 -pady 4 -relief ridge
    pack  $wn.0.l -side top -fill x -expand yes

    # List of groups

    frame     $wn.g
    label     $wn.g.la -text "$m(group_select)"
    listbox   $wn.g.lb -height 8 -yscrollcommand "$wn.g.sy set" \
                       -width 10 -xscrollcommand "$wn.g.sx set"
    scrollbar $wn.g.sy -command "$wn.g.lb yview"
    scrollbar $wn.g.sx -command "$wn.g.lb xview" -orient horizontal

    grid $wn.g.la -row 1 -column 0 -rowspan 1 -columnspan 2 -sticky ew
    grid $wn.g.lb -row 3 -column 0 -rowspan 1 -columnspan 1 -sticky ewns
    grid $wn.g.sy -row 3 -column 1 -rowspan 1 -columnspan 1 -sticky ns
    grid $wn.g.sx -row 4 -column 0 -rowspan 1 -columnspan 1 -sticky ew
    grid rowconfigure    $wn.g 3 -weight 1
    grid columnconfigure $wn.g 0 -weight 1

    # spare space

    label $wn.x -text "" -pady 0

    # List of fax numbers

    frame $wn.l
    frame $wn.l.l
    label $wn.l.l.0 -text $m(group_name)  -width 16 -anchor w
    label $wn.l.l.1 -text $m(fax_number)  -width 16 -anchor w
    label $wn.l.l.2 -text $m(recipient)   -width 16 -anchor w -padx 4
    label $wn.l.l.3 -text $m(reference)   -width 16 -anchor w -padx 4
    label $wn.l.l.4 -text $m(description) -width 16 -anchor w -padx 4
    pack $wn.l.l.0 $wn.l.l.1 $wn.l.l.2 $wn.l.l.3 $wn.l.l.4 -side left -fill x

    text $wn.l.t -width 80 -height 10 -state disabled \
         -tabs {4c left 8c left 12c left 16c left} -wrap none \
         -xscroll "$wn.l.x set" -yscroll "$wn.l.y set" \
         -relief sunken -font $fax(f1)
    mscroll $wn.l.t
    scrollbar $wn.l.y -command "$wn.l.t yview"
    scrollbar $wn.l.x -command "$wn.l.t xview" -orient horizontal

    grid $wn.l.l  -row 0 -column 0 -rowspan 1 -columnspan 2 -sticky w
    grid $wn.l.t  -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.l.y  -row 1 -column 1 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.l.x  -row 2 -column 0 -rowspan 1 -columnspan 1 -sticky news
    grid rowconfigure    $wn.l 1 -weight 1
    grid columnconfigure $wn.l 0 -weight 1

    # Group Entries

    frame $wn.j    -bd 2 -relief ridge
    label $wn.j.1  -text $m(group_name)  -anchor e -width 10
    label $wn.j.2  -text $m(description) -anchor e -width 10
    entry $wn.j.3  -textvariable fax(tmpgroupname) -width 20 \
                   -font $fax(f1) -fg $fax(c1)
    entry $wn.j.4  -textvariable fax(tmpgroupdescription) -width 40 \
                   -font $fax(f1) -fg $fax(c1)
    button $wn.j.a -text $m(clear) -width 6 -pady 0 -command {
                   fax_pbook_group_clear
    }
    button $wn.j.b -text $m(add)  -width 6 -pady 0 -command {
                   fax_pbook_group_add 
                   set pbook_modified_flag 1
    }
    button $wn.j.d -text $m(remove) -width 6 -pady 0 -command {
                   fax_pbook_group_remove 
    }
    button $wn.j.c -text $m(replace) -width 6 -pady 0 -command {
                   fax_pbook_group_replace
    }
    button $wn.j.e -text $m(sort) -width 6 -pady 0 -command {
                   fax_pbook_group_sort
    }

    grid $wn.j.1  -row 1 -column 1 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.j.2  -row 2 -column 1 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.j.3  -row 1 -column 2 -rowspan 1 -columnspan 2 -sticky news
    grid $wn.j.4  -row 2 -column 2 -rowspan 1 -columnspan 2 -sticky news
    grid $wn.j.a  -row 1 -column 4 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.j.b  -row 1 -column 5 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.j.c  -row 2 -column 5 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.j.d  -row 1 -column 6 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.j.e  -row 2 -column 6 -rowspan 1 -columnspan 1 -sticky news
    grid columnconfigure $wn.j 2 -weight 1

    # Fax Entries and Add and Edit buttons

    frame $wn.i   -bd 2 -relief ridge
    label $wn.i.1 -text $m(fax_number)  -anchor e -width 10
    label $wn.i.2 -text $m(recipient)   -anchor e -width 10
    label $wn.i.3 -text $m(reference)   -anchor e -width 10
    label $wn.i.4 -text $m(description) -anchor e -width 10

    entry $wn.i.5 -textvariable fax(tmpnumber) \
          -width 20 -font $fax(f1) -fg $fax(c1)
    entry $wn.i.6 -textvariable fax(tmprecipient) \
          -width 20 -font $fax(f1) -fg $fax(c1)
    entry $wn.i.7 -textvariable fax(tmpreference) \
          -width 20 -font $fax(f1) -fg $fax(c1)
    entry $wn.i.8 -textvariable fax(tmpdescription) \
          -width 20 -font $fax(f1) -fg $fax(c1)

    button $wn.i.a -text $m(clear)   -width 6 -pady 0 -command {
                   fax_pbook_faxnum_clear
    }
    button $wn.i.b -text $m(add)     -width 6 -pady 0 -command {
                   fax_pbook_faxnum_add
    } 
    button $wn.i.d -text $m(remove)  -width 6 -pady 0 -command {
                   fax_pbook_faxnum_remove
    }
    button $wn.i.c -text $m(replace) -width 6 -pady 0 -command {
                   fax_pbook_faxnum_replace
    }
#    button $wn.i.e -text $m(sort)    -width 6 -pady 0 -command {
#                   fax_pbook_faxnum_sort
#    }
    menubutton $wn.i.e -text $m(sort) -width 6 -pady 0 -relief raised \
                       -menu $wn.i.e.sub -takefocus 1 -highlightthickness 2

    button $wn.i.f -text "$m(merge_phone_book) ..." -width 18 -pady 0 -command {
                   fax_pbook_merge_win $wn
    }

    grid $wn.i.1  -row 1 -column 1 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.i.2  -row 2 -column 1 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.i.3  -row 3 -column 1 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.i.4  -row 4 -column 1 -rowspan 1 -columnspan 1 -sticky news

    grid $wn.i.5  -row 1 -column 2 -rowspan 1 -columnspan 2 -sticky news
    grid $wn.i.6  -row 2 -column 2 -rowspan 1 -columnspan 2 -sticky news
    grid $wn.i.7  -row 3 -column 2 -rowspan 1 -columnspan 2 -sticky news
    grid $wn.i.8  -row 4 -column 2 -rowspan 1 -columnspan 2 -sticky news

    grid $wn.i.a  -row 2 -column 4 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.i.b  -row 2 -column 5 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.i.c  -row 3 -column 5 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.i.d  -row 2 -column 6 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.i.e  -row 3 -column 6 -rowspan 1 -columnspan 1 -sticky news
    grid $wn.i.f  -row 4 -column 4 -rowspan 1 -columnspan 3 -sticky news
    grid columnconfigure $wn.i 2 -weight 1

    menu $wn.i.e.sub -tearoff no
    $wn.i.e.sub add command -label "$m(group)" -command {
                fax_pbook_faxnum_sort 0
    }
    $wn.i.e.sub add command -label "$m(fax_number)" -command {
                fax_pbook_faxnum_sort 2
    }
    $wn.i.e.sub add command -label "$m(recipient)" -command {
                fax_pbook_faxnum_sort 3
    }
    $wn.i.e.sub add command -label "$m(reference)" -command {
                fax_pbook_faxnum_sort 4
    }
    $wn.i.e.sub add command -label "$m(description)" -command {
                fax_pbook_faxnum_sort 5
    }

    # SELECT / HELP / CANCEL / CLOSE Buttons

    frame  $wn.b   -bd 10
    button $wn.b.1 -text $m(use_this_number) -pady 2 -command {
           if { $fax(tmpnumber) == "" } {
               set q [tk_dialog $wn.dialog "TkFax Dialog" \
                   "$m(no_fax_number_selected)" {error} 0 $m(ok)]
           } else { 
               fax_pbook_save_faxnumberlist
               fax_pbook_select_faxnumber
               destroy $fax(phonebookwin)
           }
    }
    button $wn.b.4 -text $m(help) -command fax_pbook_help -pady 2
    #button $wn.b.5 -text "$m(help)(HTML)" -pady 2 \
    #               -command { fax_help_browse PhoneNumber_Help.html }
    button $wn.b.2 -text $m(cancel) -pady 2 -command {
	if {$pbook_modified_flag == 1} {
            fax_pbook_cancel
	} else {
            fax_pbook_close
        }
    }
    button $wn.b.3 -text $m(close)  -pady 2 \
	           -command { fax_pbook_save_faxnumberlist
                              fax_pbook_close
	           }
    pack $wn.b.3 $wn.b.2 $wn.b.4 $wn.b.1 \
         -side right -fill x -expand yes

    # Packing of all frames

    pack $wn.b $wn.i $wn.j -side bottom -fill x
    pack $wn.0             -side top    -fill x 
    pack $wn.g $wn.l       -side left   -fill both -expand yes

    # Set the globally accessed windows

    set fax(grouplistwin)  $wn.g.lb
    set fax(numberlistwin) $wn.l.t
    set fax(tmpnumwin)     $wn.i

    # Key and Mouse bindings

    bind $wn.i.5 <KeyPress-Return> {focus $fax(phonebookwin).i.6}
    bind $wn.i.6 <KeyPress-Return> {focus $fax(phonebookwin).i.7}
    bind $wn.i.7 <KeyPress-Return> {focus $fax(phonebookwin).i.8}
    bind $wn.i.8 <KeyPress-Return> fax_pbook_faxnum_add
    bind $wn.i.5 <KeyPress-Down>   {focus $fax(phonebookwin).i.6}
    bind $wn.i.6 <KeyPress-Down>   {focus $fax(phonebookwin).i.7}
    bind $wn.i.7 <KeyPress-Down>   {focus $fax(phonebookwin).i.8}
    bind $wn.i.6 <KeyPress-Up>     {focus $fax(phonebookwin).i.5}
    bind $wn.i.7 <KeyPress-Up>     {focus $fax(phonebookwin).i.6}
    bind $wn.i.8 <KeyPress-Up>     {focus $fax(phonebookwin).i.7}
    bind $wn.j.3 <KeyPress-Return> {focus $fax(phonebookwin).j.4}
    bind $wn.j.4 <KeyPress-Return>  fax_pbook_group_add
    bind $wn.j.3 <KeyPress-Down>   {focus $fax(phonebookwin).j.4}
    bind $wn.j.4 <KeyPress-Up>     {focus $fax(phonebookwin).j.3}
    bind $fax(numberlistwin) <Button-1> fax_pbook_faxnum_setcur
    bind $fax(numberlistwin) <Button-3> fax_pbook_faxnum_unsetcur
    set  pbookwindow2 [list $wn]
    bind $fax(numberlistwin) <Double-Button-1> {
                              fax_pbook_save_faxnumberlist
                              fax_pbook_select_faxnumber
	                      wm withdraw $pbookwindow2  ;# hide window
    }
    bind $fax(grouplistwin)  <ButtonRelease-1> {fax_pbook_group_setcur %x %y}
    bind $fax(grouplistwin)  <Button-3>      fax_pbook_group_unsetcur
#    bind $fax(grouplistwin)  <Double-Button> fax_pbook_select_group

    wm protocol $wn WM_DELETE_WINDOW {
        fax_pbook_save_faxnumberlist
        fax_pbook_close
    }

    # Initialization

    set fax(curgroupname)	 "All"
    set fax(tmpgroupname)	 "All"
    set fax(tmpgroupdescription) "Fax numbers from all groups"
    set fax(tmpnumber)      	 ""
    set fax(tmprecipient)   	 ""
    set fax(tmpreference)   	 ""
    set fax(tmpdescription) 	 ""
    set fax(group_and_number)	 ""

    set pbook_sort_order(0) 0
    set pbook_sort_order(1) 0
    set pbook_sort_order(2) 0
    set pbook_sort_order(3) 0
    set pbook_sort_order(4) 0
    set pbook_sort_order(5) 0
    set pbook_grpsort_order 0
    set pbook_modified_flag 0
    set fax(removed_grouplist) {}
    set fax(group_selno)    0

    $fax(grouplistwin) selection set 0
    fax_pbook_display_grouplist
    $fax(grouplistwin) selection set 0
    fax_pbook_display_faxnumberlist All
}

#
#  Merge pbook
proc fax_pbook_merge_win {w} {
    global fax m
    # Name of this window
    set wm $fax(phonebookwin).merge
    if [winfo exists $wm] {return}

    toplevel $wm
    wm title $wm "TkFax: Merging Number Book"

    set oldpbook $fax(olduserdir)/pbook/group.pbd

    # Top label

    label $wm.l -text "Merging Phone Book:" -padx 20 -pady 4 -relief ridge
    frame $wm.1
    entry $wm.1.ent -width 32
    if [file exist $oldpbook] {
          $wm.1.ent insert  0 $oldpbook
    }
    button $wm.1.b -text "$m(select)..." -command {
	set pbook2 [tk_getOpenFile -defaultextension pbd \
         -filetypes { {"Phone book" .pbd} {"All files" *} } \
         -initialdir $fax(curdir) -initialfile group.pbd -parent $fax(phonebookwin).merge -title "Select Phone Book file"]
        if { "$pbook2" != "" } {
	    $fax(phonebookwin).merge.1.ent delete 0 end
	    $fax(phonebookwin).merge.1.ent insert  0 $pbook2
	}
    }
    pack  $wm.1.ent $wm.1.b -side left -fill x -expand yes
    frame $wm.2
    button $wm.2.b1 -text $m(excute)  -pady 2 \
       -command {fax_pbook_merge [$fax(phonebookwin).merge.1.ent get]
                 destroy $fax(phonebookwin).merge}
    button $wm.2.b2 -text $m(cancel) -pady 2 -command {destroy $fax(phonebookwin).merge}
    pack $wm.2.b2 $wm.2.b1 \
         -side right -fill x -expand yes
    pack $wm.l $wm.1 $wm.2 -side top -fill x -expand yes
#fax_pbook_read_faxnumberlist
}
#
# Procedures for phone book window
#

proc fax_pbook_read_faxnumberlist {} {
#
# This procedure reads group list file into the variable fax(grouplist).
# Then it reads each group file into the variable fax(group-$groupname).
#
#  8 Feb 2005 Masaki Shinomiya: not use fax(pbookgroupfile)
# 29 Sep 2002 modified by R.Kawagishi
#
    global fax m
    global pbook_modified_flag

    # Read group list into a variable fax(grouplist).
    set groupfile  $fax(pbookdir)/group.pbd
    set fax(grouplist) {}
    if [file exist $groupfile] {
        set fid [open $groupfile r]
        while {[set line [gets $fid]]!={}} {
            lappend fax(grouplist) "$line"
        }
    }

    # Read fax number list into variables fax(group-$groupname)

    set fax(group_and_number) {}
    foreach group $fax(grouplist) {
        set groupname [lindex [split $group \t] 0]
        set groupdesc [list [lreplace $group 0 0]]
        set fax(group-$groupname) {}
        set file $fax(pbookdir)/$groupname.pbd
        if [file exist $file] {
            set fid [open $file r]
            while { [set line [gets $fid]] != {} } {
                set work [split $line \t]
                lappend fax(group-$groupname) $work
                lappend fax(group_and_number) $groupname\t$groupdesc\t$work
            }

        } else {
            puts "TkFax: Fax list file $file $m(does_not_exist)"
        }
    }
    set fax(removed_grouplist) {}
    set pbook_modified_flag 0
}

#  9 Feb 2005  Masaki Shinomiya
proc fax_pbook_merge { pbook2 } {
    global fax m
    global pbook_modified_flag
    set appendflag 0
    fax_debug "fax_pbook_merge is called with $pbook2."
    if [file isdirectory "$pbook2"] {
	set groupfile  $pbook2/group.pbd
	set pbookdir $pbook2
    } else {
	set groupfile $pbook2
	set pbookdir [file dirname $pbook2]
    }
    # Read group list into a variable fax(grouplist).
    if [file exist $groupfile] {
        set fid [open $groupfile r]
        while {[set line [gets $fid]]!={}} {
            set groupname [lindex [split $line \t] 0]
            set groupdesc [list [lreplace $line 0 0]]
            if { [lsearch $fax(grouplist) "${groupname}\t*"] < 0 } {
		fax_debug "$groupname is new group."
                lappend fax(grouplist) "$line"
        	set fax(group-$groupname) {}
		set appendflag 1
	    } else {
		fax_debug "$groupname is already exists."
	    }

	    # Read fax number
        set file $pbookdir/$groupname.pbd
        if [file exist $file] {
            set fid2 [open $file r]
            while { [set line [gets $fid2]] != {} } {
                set work [split $line \t]
		set number [lrange $work 0 1]
		if {[lsearch $fax(group-$groupname) "$number *" ] < 0 } {
		    fax_debug "$work is new entry."
                #puts "$groupname\t$groupdesc\t$work"
                lappend fax(group-$groupname) $work
                lappend fax(group_and_number) $groupname\t$groupdesc\t$work
		set appendflag 1
		} else {
		    fax_debug "$work already exist."
		}
            }
        } else {
            fax_debug "TkFax: Fax number list file $file $m(does_not_exist)"
        }

        }
    }
    if $appendflag {
        $fax(grouplistwin) selection set 0
        fax_pbook_display_grouplist
        $fax(grouplistwin) selection set 0
        fax_pbook_display_faxnumberlist All
        set pbook_modified_flag 1
    }
    return $appendflag
}
proc fax_pbook_save_faxnumberlist {} {
#
# This saves fax number list stored in fax(group-groupname) into a file
# $fax(pbookdir)/groupname.pbd, if changes are made.
#
# 12 Dec 2002 look up pbook_modified_flag (M.S)
# 26 Sep 2002 modified by R.Kawagishi
#
    global fax
    global pbook_modified_flag

    if { $pbook_modified_flag == 0 } { return }

    foreach groupname $fax(removed_grouplist) {
        set file "$fax(pbookdir)/$groupname.pbd"
	if [file exists $file] {
	    file copy -force -- $file $file.bak
            file delete -- $file
	}
    }
    set fax(removed_grouplist) {}

    set gfile "$fax(pbookdir)/group.pbd"
    if [file exists $gfile] {
	    file copy -force -- $gfile $gfile.bak
    }
    set gid [open $gfile w]
    foreach group $fax(grouplist) {
        puts $gid $group
        set groupname [lindex $group 0]
        set file "$fax(pbookdir)/$groupname.pbd"
	if [file exists $file] {
	    file copy -force -- $file $file.bak
	}
        set fid [open $file w]
        foreach member $fax(group-$groupname) {
#DEBUG        puts $fid $member
            set member2 [join $member \t]
            puts $fid $member2
        }
        close $fid
    }
    close $gid

    set pbook_modified_flag 0
}

proc fax_pbook_display_grouplist {} {
#
# This displays group list on $fax(grouplistwin),
#
# 14 Sep 2002 modified by R.Kawagishi
#
    global fax

    $fax(grouplistwin) delete 0 end
    $fax(grouplistwin) insert end "All"
    foreach item $fax(grouplist) {
        set grpname [lindex $item 0]
        $fax(grouplistwin) insert end "$grpname"
    }
}

proc fax_pbook_group_setcur {x y} {
#
# This shows the group name and description selected by mouse.
#
# 19 Sep 2002 modified by R.Kawagishi
#
    global fax
    global focused_grp_num focused_grp_name

    set elements [$fax(grouplistwin) get @$x,$y]
    set fax(curgroupname) [lindex $elements 0]

    set focused_grp_num  [$fax(grouplistwin) curselection]
    set focused_grp_name [$fax(grouplistwin) get $focused_grp_num]

    fax_pbook_display_group  [$fax(grouplistwin) curselection]
    fax_pbook_display_faxnumberlist $fax(tmpgroupname)
}

proc fax_pbook_group_unsetcur {} {
    global fax

#    $fax(grouplistwin) tag delete cur
    $fax(grouplistwin) selection clear 0 end
    set fax(tmpgroupname) ""
    set fax(tmpgroupdescription) ""
}

proc fax_pbook_display_group { grp_selno } {
    global fax

    set fax(group_selno) $grp_selno
    if {$fax(group_selno) == 0} {
        set fax(curgroupname) "All"
        set fax(tmpgroupname) "All"
        set fax(tmpgroupdescription) "Fax numbers from all groups"
    } else {
        set grpdata [lindex $fax(grouplist) [expr $fax(group_selno) -1] ]
        set fax(curgroupname) [lindex $grpdata 0]
        set fax(tmpgroupname) [lindex $grpdata 0]
        set fax(tmpgroupdescription) [lreplace $grpdata 0 0]
    }
}

proc fax_pbook_group_clear {} {
    global fax
    fax_message ""
    set fax(tmpgroupname)	 ""
    set fax(tmpgroupdescription) ""
    focus $fax(phonebookwin).j.3
}

proc fax_pbook_group_sort {} {
#
# 14 Sep 2002 create by R.Kawagishi
#
    global fax pbook_grpsort_order
    global focused_grp_num focused_grp_name

    fax_message ""
#    set focused_grp_num  [$fax(grouplistwin) curselection]
#    set focused_grp_name [$fax(grouplistwin) get $focused_grp_num]
    if { $pbook_grpsort_order == 0 } {
        set fax(grouplist) [lsort -increasing -dictionary -index 0 $fax(grouplist)]
    } else {
        set fax(grouplist) [lsort -decreasing -dictionary -index 0 $fax(grouplist)]
    }
    set pbook_grpsort_order [expr ~$pbook_grpsort_order]

    $fax(grouplistwin) delete 0 end
    $fax(grouplistwin) insert end "All"
    set i 1
    foreach item $fax(grouplist) {
        set grpname [lindex $item 0]
        $fax(grouplistwin) insert end "$grpname"
        if {$grpname == $focused_grp_name} {
            set focused_grp_num $i
        }
        incr i
    }
    if {$grpname == "All"} {
        set focused_grp_num 0
    }
    $fax(grouplistwin) selection set $focused_grp_num
    $fax(grouplistwin) see $focused_grp_num
}

proc fax_pbook_groupname_check { groupname mode } {
    global fax m wn
    # mode: add|remove
    # check groupname to see it is non-empty or already used.
    if {$groupname==""} { return 1 }
    foreach reserved_group {All Misc} {
        if { $groupname == $reserved_group } {
            set q [tk_dialog $wn.dialog "TkFax Dialog" \
                  "$m(group) $reserved_group $m(is_reserved)"  {error} 0 $m(ok)]
            return 1
        }
    }

    set found 0
    foreach igroup $fax(grouplist) {
        set igroupname [lindex [split $igroup \t] 0]
        if {$groupname==$igroupname} {
	    set found 1
	    if {$mode == "add" } {
        	set q [tk_dialog $wn.dialog "TkFax Dialog" \
                   "$m(group_name) $groupname $m(is_already_used)" \
                   {error} 0 $m(ok)]
        	return 1
	    } 
        }
    }
    if {$mode != "add" && $found == 0 } {
        	set q [tk_dialog $wn.dialog "TkFax Dialog" \
                   "$m(group_name) $groupname $m(does_not_exist) " \
                   {error} 0 $m(ok)]
        	return 1
    }
    return 0
}

proc fax_pbook_group_add {} {
#
# 14 Sep 2002 modified by R.Kawagishi
#
    global fax m pbook_modified_flag wn
    global focused_grp_num focused_grp_name

    fax_message ""
    set fax(tmpgroupname) [string trim $fax(tmpgroupname)]
    set groupname $fax(tmpgroupname)
#    set replace 0
    # check groupname to see it is non-empty or already used.
    if [fax_pbook_groupname_check $groupname add] {return 1}

    # Add a line to a list and create a new list variable.

    if {$fax(tmpgroupdescription) == ""} {
        set fax(tmpgroupdescription) "(none)"
    }
    set line "$groupname\t$fax(tmpgroupdescription)"
#    $fax(grouplistwin) configure -state normal
    $fax(grouplistwin) insert end $groupname
    $fax(grouplistwin) yview end
#    $fax(grouplistwin) configure -state disabled
    lappend fax(grouplist) $line
#    lappend fax(group_and_number) $line
    set fax(group-$groupname) {}

    # Check whether database file already exists. If so, try to read it in.

    set file $fax(pbookdir)/$groupname.pbd
    if [file exist $file] {
        set q [tk_dialog $wn.dialog "TkFax Dialog" \
  "$m(phone_book_database_file) $file $m(is_already_exists) $m(do_you_want_to_read_it_in)" \
               {question} 0 $m(yes) $m(no) ]
        if {$q==0} {
            set fid [open $file r]
            while {[set line [gets $fid]]!={}} {
                lappend fax(group-$groupname) "$line"
            }
        }
    }

    #set m(is_added) "is added"

    fax_pbook_display_grouplist
    set grp_lb_no [llength $fax(grouplist)]
    set fax(group_selno)    $grp_lb_no
    fax_pbook_display_group $grp_lb_no
    $fax(grouplistwin) selection set $grp_lb_no
    $fax(grouplistwin) see $grp_lb_no
    set focused_grp_num  $grp_lb_no
    set focused_grp_name $groupname
    fax_pbook_display_faxnumberlist $groupname
    fax_message "Phone book: $m(group_name) '$groupname' $m(is_added)"
    set pbook_modified_flag 1

    # Ready for another addition.

    focus $fax(phonebookwin).j.3
}

proc fax_pbook_group_remove {} {
# 
# 26 Sep 2001 modified by Masaki Shinomiya
# 14 Sep 2002 modified by R.Kawagishi
#
    global fax m pbook_modified_flag wn
    global focused_grp_num focused_grp_name

    fax_message ""
    set fax(tmpgroupname) [string trim $fax(tmpgroupname)]
    set groupname $fax(tmpgroupname)

    # check groupname to see it is non-empty or reserved.
    if [fax_pbook_groupname_check $groupname remove] {return 1}
    set file $fax(pbookdir)/$groupname.pbd
    set q [tk_dialog $wn.dialog "TkFax Dialog" \
           "$m(to_delete_group) '$groupname' - $m(is_it_ok)"\
           {question} 1 $m(yes) $m(no) ]
    if {$q==0} {
        set i 0
        foreach groupdata $fax(grouplist) {
            incr i
            if {[lindex $groupdata 0] == $groupname} {
                set grp_no $i
            }
        }
        set glist_no [expr $grp_no -1]
	set new_glist [lreplace $fax(grouplist) $glist_no $glist_no]
	set fax(grouplist) $new_glist
 	fax_pbook_display_grouplist

        set fax(group_selno) 0
        $fax(grouplistwin) selection set 0
        set focused_grp_num  0
        set focused_grp_name "All"

	fax_pbook_display_faxnumberlist All
        set fax(tmpgroupname) ""
        set fax(tmpgroupdescription) ""
        unset fax(group-$groupname)
        # exec rm -f $file
	lappend fax(removed_grouplist) $groupname
        fax_message \
            "Phone book: $m(group) '$groupname' $m(deleted)"
        set pbook_modified_flag 1
    }
}

proc fax_pbook_group_replace {} {
#
# 14 Sep 2002 creat by R.Kawagishi
#
    global fax m pbook_modified_flag wn
    global focused_grp_num focused_grp_name

    fax_message ""
    set grp_name     $fax(curgroupname)
    set fax(tmpgroupname) [string trim $fax(tmpgroupname)]
    set new_grp_name $fax(tmpgroupname)

    #if { $new_grp_name == "" } { return 1}
    # check groupname to see it is non-empty or reserved.
    if [fax_pbook_groupname_check $grp_name remove] {return 1}
    if {$new_grp_name != $grp_name} {
      # check groupname to see it is non-empty or already used.
      if [fax_pbook_groupname_check $new_grp_name add] {return 1}
    }

#    set grp_lb_no [$fax(grouplistwin) curselection]
    set grp_lb_no $focused_grp_num
    if { [llength $grp_lb_no] != 1 } {
	fax_debug "grouplistwin curselection = $grp_lb_no"
        set q [tk_dialog $wn.dialog "TkFax Dialog" \
            "$m(select_group_other_than_all)"  {error} 0 $m(ok)]
	return 1
    }
    set glist_no  [expr $grp_lb_no -1]
    set grp_desc  [lindex [lindex $fax(grouplist) $glist_no] 1]

    set q [tk_dialog $wn.dialog "TkFax Dialog" \
"$m(group_name)$grp_name
$m(description)$grp_desc
-->
$m(group_name)$new_grp_name
$m(description)$fax(tmpgroupdescription)
$m(confirm_change)" \
    {question} 0 $m(ok) $m(cancel)]
    if {$q == 0} {
        set grp_dat   "$new_grp_name\t$fax(tmpgroupdescription)"
        set new_glist [lreplace $fax(grouplist) $glist_no $glist_no $grp_dat]
        set fax(grouplist) $new_glist
        set fax(group-$new_grp_name) $fax(group-$grp_name)
        if {$new_grp_name != $grp_name} {
            unset fax(group-$grp_name)
	    lappend fax(removed_grouplist) $grp_name
        }
        set fax(tmpgroupname) ""
        set fax(tmpgroupdescription) ""
        $fax(grouplistwin) delete 0 end
        $fax(grouplistwin) insert end "All"
        foreach item $fax(grouplist) {
            $fax(grouplistwin) insert end [lindex $item 0]
        }
	fax_pbook_display_grouplist
        set fax(group_selno)    $grp_lb_no
        fax_pbook_display_group $grp_lb_no
        $fax(grouplistwin) selection set $grp_lb_no
        $fax(grouplistwin) see $grp_lb_no
	fax_pbook_display_faxnumberlist $new_grp_name
        fax_message "Phone book: $m(group_name) $new_grp_name $m(is_updated)"
        set pbook_modified_flag 1
    }
}

proc fax_pbook_select_group {} {
#
# This select a group for faxing.
#
    global fax
    set fax(number)      group:$fax(tmpgroupname)
    set fax(recipient)   $fax(tmpgroupdescription)
    destroy $fax(phonebookwin)
}

proc fax_pbook_display_faxnumberlist { groupname } {
#
# This displays fax number list of a selected group on $fax(numberlistwin).
#
# 19 Sep 2002 modified by R.Kawagishi
#
    global fax pbook_sort_order

    $fax(numberlistwin) configure -state normal
    $fax(numberlistwin) delete 1.0 end
    set fax(group_and_number) ""

    if { $groupname == "All" } {
        set fax(group-All) {}
        foreach group $fax(grouplist) {
            set groupname [lindex $group 0]
            set groupdesc [list [lreplace $group 0 0]]
            foreach member $fax(group-$groupname) {
                lappend fax(group-All) "$groupname\t[join $member \t]"
                lappend fax(group_and_number) $groupname\t$groupdesc\t[join $member \t]
            }
        }

#        if { $pbook_sort_order == 0 } {
#            set fax(group-All) \
#            [lsort -increasing -command fax_recipient_compare $fax(group-All)]
#        } else {
#            set fax(group-All) \
#            [lsort -decreasing -command fax_recipient_compare $fax(group-All)]
#        }
        foreach member $fax(group-All) {
            $fax(numberlistwin) insert end "$member\n"
        }
    } else {
        set groupdesc [list $fax(tmpgroupdescription)]
        foreach member $fax(group-$groupname) {
            $fax(numberlistwin) insert end "$groupname\t[join $member \t]\n"
            lappend fax(group_and_number) "$groupname\t$groupdesc\t[join $member \t]"
        }
    }
    $fax(numberlistwin) delete {end -1 char}
    $fax(numberlistwin) configure -state disabled
}

proc fax_pbook_faxnum_setcur {} {
#
# This displays faxnumber element on entry windows when left mouse button is
# pressed on $fax(numberlistwin) and deselect it when right mouse button is
# pressed.
#
# 14 Sep 2002 modified by R.Kawagishi
#
    global fax

    $fax(numberlistwin) tag delete cur
    $fax(numberlistwin) tag configure cur -foreground red -background skyblue
    $fax(numberlistwin) tag add cur \
                                {current linestart} {current lineend +1 char}
    set line [$fax(numberlistwin) get {current linestart} {current lineend}]
    set elements [split $line \t]

    if {$fax(group_selno) > 0} {
        set grpdata [lindex $fax(grouplist) [expr $fax(group_selno) -1] ]
        set fax(curgroupname) [lindex $grpdata 0]
        set fax(tmpgroupname) [lindex $grpdata 0]
#        set fax(tmpgroupdescription) [lindex $grpdata 1]
        set fax(tmpgroupdescription) [lreplace $grpdata 0 0]
    } else {
        set fax(curgroupname) [lindex $elements 0]
        set fax(tmpgroupname) [lindex $elements 0]
        regexp {^.} [$fax(numberlistwin) index current] line_number
        set work [lindex $fax(group_and_number) [expr $line_number -1] ]
        set fax(tmpgroupdescription) [lindex $work 1]
    }
    set fax(tmpnumber)      [lindex $elements 1]
    set fax(tmprecipient)   [lindex $elements 2]
    set fax(tmpreference)   [lindex $elements 3]
    set fax(tmpdescription) [lindex $elements 4]
}

proc fax_pbook_faxnum_unsetcur {} {
    global fax
    $fax(numberlistwin) tag delete cur
    set fax(tmpnumber)      ""
    set fax(tmprecipient)   ""
    set fax(tmpreference)   ""
    set fax(tmpdescription) ""
}

#
# These two procedures sort the faxnumberlist.

proc fax_pbook_faxnum_sort {item} {
    global fax pbook_sort_order

    fax_message ""
    set newlist1 ""
    foreach line $fax(group_and_number) {
        set orglist  [split $line \t]
        set work2    [lindex $orglist 2]
        set sho_num  [num_shorten $work2]
        set new_line [lappend orglist $work2]
        set new_line [lreplace $new_line 2 2 $sho_num]
        lappend newlist1 $new_line
    }

    if { $pbook_sort_order($item) == 0 } {
        set newlist2 \
        [lsort -increasing -dictionary -index $item $newlist1]
    } else {
        set newlist2 \
        [lsort -decreasing -dictionary -index $item $newlist1]
    }
    set pbook_sort_order($item) [expr ~$pbook_sort_order($item)]

    set fax(group_and_number) ""
    foreach line $newlist2 {
        set new_line [lreplace $line 2 2 [lindex $line 6]]
        set new_line [lreplace $new_line 6 6]
        lappend fax(group_and_number) $new_line
    }

    $fax(numberlistwin) configure -state normal
    $fax(numberlistwin) delete 1.0 end
    set newlist ""

    foreach line $fax(group_and_number) {
        $fax(numberlistwin) insert end [lindex $line 0]\t
        $fax(numberlistwin) insert end [lindex $line 2]\t
        $fax(numberlistwin) insert end [lindex $line 3]\t
        $fax(numberlistwin) insert end [lindex $line 4]\t
        $fax(numberlistwin) insert end [lindex $line 5]\n
        lappend newlist [lindex $line 0]\t[lindex $line 1]\t[lindex $line 2]\t[lindex $line 3]\t[lindex $line 4]\t[lindex $line 5]
    }
    set fax(group_and_number) $newlist
    $fax(numberlistwin) configure -state disabled
}

#proc fax_recipient_compare { a b } {
#    set x [lindex [split $a \t] 1]
#    set y [lindex [split $b \t] 1]
#    return [string compare $x $y]
#}

proc fax_pbook_faxnum_clear {} {
#
# This clears entry windows.
#
    global fax
    set fax(tmpnumber)      ""
    set fax(tmprpbook_modified_flagecipient)   ""
    set fax(tmprecipient)   ""
    set fax(tmpreference)   ""
    set fax(tmpdescription) ""
    focus $fax(phonebookwin).i.5
}

proc fax_pbook_faxnum_check { faxnumber } {
    global fax m

    set number [string trim $faxnumber]
    if { $number == {} } {
        tk_messageBox -message $m(enter_fax_number) -icon error -type ok
        return 1
    }

    if { [num_shorten $number] == "-" } {
        tk_messageBox -message "$m(fax_number_illegal):\n$number" -icon error -type ok
        return 1
    }

    return 0
}

proc fax_pbook_faxnum_add {} {
#
# This adds a new fax number to the current group.
#
# 12 Dec 2002 set pbook_modified_flag (M.S)
# 16 Sep 2002 modified by R.Kawagishi
#
    global fax m
    global pbook_modified_flag wn

    fax_message ""
    set fax(tmpnumber) [string trim $fax(tmpnumber)]
    if [fax_pbook_faxnum_check $fax(tmpnumber)] {return 1}
    set groupname $fax(curgroupname)
    set grp_lb_no [$fax(grouplistwin) curselection]
    if { [llength $grp_lb_no] != 1 } {
	fax_debug "grouplistwin curselection = $grp_lb_no"
        set q [tk_dialog $wn.dialog "TkFax Dialog" \
            "$m(select_group_other_than_all)"  {error} 0 $m(ok)]
	return 1
    }
    set glist_no  [expr $grp_lb_no -1]
    set grp_desc  [lindex [lindex $fax(grouplist) $glist_no] 1]
    if {$groupname== "All"} {
        set groupname "Misc"
	set grp_desc  [list "Miscellaneous fax numbers"]
    }

    if {$fax(tmprecipient)   == ""} {set fax(tmprecipient)   "(none)"}
    if {$fax(tmpreference)   == ""} {set fax(tmpreference)   "(none)"}
    if {$fax(tmpdescription) == ""} {set fax(tmpdescription) "(none)"}

    $fax(numberlistwin) configure -state normal
    set line1 "$fax(tmpnumber)\t$fax(tmprecipient)\t$fax(tmpreference)\t$fax(tmpdescription)"
    set line2 [list $fax(tmpnumber) $fax(tmprecipient) $fax(tmpreference) $fax(tmpdescription)]

    set insertpoint [$fax(numberlistwin) index end]
    $fax(numberlistwin) insert end "\n$groupname\t$line1"

    if {[$fax(numberlistwin) get 1.0]=="\n"} {
        $fax(numberlistwin) delete 1.0
    }
    $fax(numberlistwin) yview end
    $fax(numberlistwin) configure -state disabled
    lappend fax(group-$groupname) $line2
    lappend fax(group_and_number) $groupname\t$grp_desc\t$line1

    fax_message \
        "Phone book: $m(fax_number) '$fax(tmpnumber)' $m(is_registered_in) $groupname group."
    fax_pbook_faxnum_clear
    focus $fax(phonebookwin).i.5
    set pbook_modified_flag 1
}

proc fax_pbook_faxnum_remove {} {
#
# This removes a fax number from the current group.
#
# 12 Dec 2002 faxnum_remove_search, pbook_modified_flag (M.S)
# 26 Sep 2001 error trap when no nummbers selected
# 14 Sep 2002 modified by R.Kawagishi
#
    global fax m
    global pbook_modified_flag wn

    fax_message ""
    set fax(tmpnumber) [string trim $fax(tmpnumber)]
    if { $fax(tmpnumber) ==  "" } { return 1 }
    set groupname $fax(curgroupname)
    if {$groupname == "All" || $groupname == ""} {
        set q [tk_dialog $wn.dialog "TkFax Dialog" \
            "$m(select_group_other_than_all)"  {error} 0 $m(ok)]
        return
        #set work3 [lreplace $fax(group_and_number) $line_num $line_num]
        #set fax(group_and_number) $work3
    }
    if { [$fax(numberlistwin) tag nextrange cur 1.0 end] == "" } {
        set q [tk_dialog $wn.dialog "TkFax Dialog" \
            "$m(select_fax_number)"  {error} 0 $m(ok)]
        return
    }
    set q [tk_dialog $wn.dialog "TkFax Dialog" \
          "$m(to_delete_fax_number) $fax(tmpnumber) $m(is_it_ok)"\
           {question} 1 $m(yes) $m(no) ]
    if {$q==0} {

        regexp {^.} [$fax(numberlistwin) index cur.first] work1
	set line_num1 [expr $work1 -1]
        set line_num2 [faxnum_remove_search $line_num1]
        set work [lreplace $fax(group_and_number) $line_num1 $line_num1]
        set fax(group_and_number) $work
        set work [lreplace $fax(group-$fax(curgroupname)) $line_num2 $line_num2]
        set fax(group-$fax(curgroupname)) $work
        $fax(numberlistwin) configure -state normal
        $fax(numberlistwin) delete cur.first cur.last
        $fax(numberlistwin) configure -state disabled
        fax_message \
            "Phone book: '$fax(tmpnumber)'  in $m(group) '$groupname' $m(is_deleted)"
	foreach i {number recipient reference description} {
	    set fax(tmp$i) ""
	}
        set pbook_modified_flag 1
    }
}

proc faxnum_remove_search { line_num } {
	# 12 Dec 2002  (M.S)
	global fax
        set work2 [split [lindex $fax(group_and_number) $line_num] \t]
	set work4 [lreplace $work2 0 1]
        set work5 [lsearch $fax(group-$fax(curgroupname)) $work4]
#DEBUG	puts "$fax(curgroupname) $work5"
	return $work5
}

proc fax_pbook_faxnum_replace {} {
#
# This replaces the selected faxnumber with corrected info.
#
# 12 Dec 2002 use faxnum_remove_search, set pbook_modified_flag (M.S)
# 26 Sep 2001 error trap when no nummbers selected
# 20 Sep 2001 check the group, replacing only one item
# 19 Sep 2002 modified by R.Kawagishi

    global fax m
    global pbook_modified_flag wn

    fax_message ""
    set fax(tmpnumber) [string trim $fax(tmpnumber)]
    if [fax_pbook_faxnum_check $fax(tmpnumber)] {return 1}
    set groupname $fax(curgroupname)
    if {$groupname == "All" || $groupname == ""} {
            set q [tk_dialog $wn.dialog "TkFax Dialog" \
                  "$m(select_group_other_than_all)"  {error} 0 $m(ok)]
        return
    }
    if { [$fax(numberlistwin) tag nextrange cur 1.0 end] == "" } {
        set q [tk_dialog $wn.dialog "TkFax Dialog" \
            "$m(select_fax_number)"  {error} 0 $m(ok)]
        return
    }

#    set glist_no [$fax(grouplistwin) curselection]
#    set grp_desc [list [lreplace [lindex $fax(grouplist) $glist_no] 0 0]]

    set grp_desc [list [lreplace [lindex $fax(grouplist) $fax(group_selno)] 0 0]]

    if {$groupname== "All"} {
        set groupname "Misc"
        set grp_desc  [list "Miscellaneous fax numbers"]
    }
    if {$fax(tmprecipient)   == ""} {set fax(tmprecipient)   "(none)"}
    if {$fax(tmpreference)   == ""} {set fax(tmpreference)   "(none)"}
    if {$fax(tmpdescription) == ""} {set fax(tmpdescription) "(none)"}

    regexp {^.} [$fax(numberlistwin) index cur.first] work
    set line_num1 [expr $work -1]
    set line_num2 [faxnum_remove_search $line_num1]
    $fax(numberlistwin) configure -state normal
    set insertpoint [$fax(numberlistwin) index cur.first]
    set endpoint    [$fax(numberlistwin) index end]

    $fax(numberlistwin) delete cur.first cur.last
    set line1 \
    "$fax(tmpnumber)\t$fax(tmprecipient)\t$fax(tmpreference)\t$fax(tmpdescription)"
    set line2 [list \
        $fax(tmpnumber) $fax(tmprecipient) $fax(tmpreference) $fax(tmpdescription)]

    if { $insertpoint < [expr $endpoint-1] } {
        $fax(numberlistwin) insert $insertpoint "$groupname\t$line1\n"
    } elseif { $insertpoint > 1 } {
        $fax(numberlistwin) insert $insertpoint "\n$groupname\t$line1"
    } else {
        $fax(numberlistwin) insert $insertpoint "$groupname\t$line1"
    }
    $fax(numberlistwin) configure -state disabled
#    fax_pbook_reset_numberlist
    set work [lreplace $fax(group-$fax(curgroupname)) $line_num2 $line_num2 $line2]
    set fax(group-$fax(curgroupname)) $work

    set work [lreplace $fax(group_and_number) $line_num1 $line_num1 \
              $groupname\t$grp_desc\t$line1]
    set fax(group_and_number) $work
    set pbook_modified_flag 1
}

proc fax_pbook_select_faxnumber {} {
#
# This selects current faxnumber for faxing.
#
    global fax m wn

    fax_message ""
    set fax(tmpnumber) [string trim $fax(tmpnumber)]
    if [fax_pbook_faxnum_check $fax(tmpnumber)] {return 1}
    set fax(number)      $fax(tmpnumber)
    set fax(recipient)   $fax(tmprecipient)
    set fax(reference)   $fax(tmpreference)
    set fax(description) $fax(tmpdescription)
#    destroy $fax(phonebookwin)
}

proc fax_pbook_cancel {} {
#
# 29 Sep 2001

    global m wn

    fax_message ""
    set q [tk_dialog $wn.dialog "TkFax Dialog" \
           "$m(abort_all_changes_and_exit) $m(is_it_ok)"\
           {question} 0 $m(yes) $m(no) ]
    if {$q==0} {
        fax_pbook_read_faxnumberlist
        fax_pbook_close
	fax_message ""
    }
}

proc fax_pbook_close {} {
#
# This closes the phone book window.
#
    global fax
    fax_message ""
#   fax_pbook_save_faxnumberlist
   destroy $fax(phonebookwin)
}

proc fax_pbook_help {} {
    global fax m
    fax_help "Phone Book Help" "$m(pbookhlp)"
}

