############################################################
# This file is a part of TkFax.
# html_viewer.tcl
#
# This tcl script support viewing HTML files.
#-----------------------------------------------------------
# Original file is sample.tcl in html_library-0.3.tar.gz ,
# by Stephen Uhler (stephen.uhler@sun.com)
#-----------------------------------------------------------
# 11 Oct 2002 I18N, mscroll, about
# 20 Sep 2002 close command 
# Modified by R.Kawagishi (kawagisi@yk.rim.or.jp)
# 26 Aug 2002

#-----------------------------------------------------------
# HTML Viewer Window
#-----------------------------------------------------------
proc html_viewer {url} {
    global fax m
    global hv_Size hv_Indent hv_Home hv_Url hv_Running hv_message url_hist_flag

    set hv_Size    4         ;# font size adjustment
    set hv_Indent  1.2       ;# tab spacing (cm)
    set hv_Home    $url      ;# home document
    set hv_Url     $url      ;# current file
    set hv_Running busy      ;# page status
    set hv_message ""        ;# message line
    set url_hist_flag 0      ;#

    #-------------------------------------------------------
    # make the interface and render the home page
    #-------------------------------------------------------
    catch setup              ;# the catch lets us re-source this file
    HMinit_win   .hv.text
    HMset_state  .hv.text -size $hv_Size
    HMset_indent .hv.text $hv_Indent
    render $hv_Home
}

#-----------------------------------------------------------
# ʸɤ߹
#-----------------------------------------------------------
proc LoadFile {} {
    set ftypes {
        {"Text File"    {.htm .html} }
        {"All files"    *}
    }
    set fname [tk_getOpenFile -filetypes $ftypes -parent .]
    if {$fname == ""} {
        return
    } else {
        render $fname
    }
}

#-----------------------------------------------------------
# construct a simple user interface
#-----------------------------------------------------------
# 2002 Aug 26 by R.Kawagishi  modified and add misc

proc setup {} {
    global fax m 

    # Name of this window

    if [winfo exists .hv] {return}
    toplevel .hv
    wm title .hv "TkFax: HTML Viewer"

    frame .hv.frame1
    frame .hv.frame2

    menubutton .hv.menu1  -relief raised -bd 2 -text $m(file)\
                          -menu .hv.menu1.f -bg skyblue
    label      .hv.file   -text "file:"
    entry      .hv.entry  -textvariable hv_Url -width 50 -bg cornsilk
    label      .hv.status -textvariable hv_Running -width 6 -relief ridge \
		       -bd 2 -padx 9 -pady 3
    menubutton .hv.menu2  -relief raised -bd 2 -text "$m(option)..." \
                          -menu .hv.menu2.m -bg skyblue
    button     .hv.help   -command {hv_help} -text "About HTML Viewer" -bg skyblue
    button     .hv.quit   -command {destroy .hv} -text "$m(close)" -bg skyblue

    label      .hv.msg -textvariable hv_message -width 80
    button     .hv.reverse -text "$m(back)" -command {reverse_url}
    button     .hv.forward -text "$m(forward)" -command {forward_url}
    button     .hv.histbtn -text "$m(history)..."   -command {help_history}

    scrollbar  .hv.yscrollbar -command ".hv.text yview" -orient v
    scrollbar  .hv.xscrollbar -command ".hv.text xview" -orient h

    option add *Text.height 40 startup
    option add *Text.width  80 startup
    text .hv.text -yscrollcommand ".hv.yscrollbar set" \
                  -xscrollcommand ".hv.xscrollbar set" -padx 3 -pady 3 -takefocus 0
    mscroll    .hv.text

    pack .hv.frame1 .hv.frame2 -side top
    pack .hv.yscrollbar -side right  -fill y    -expand 0
    pack .hv.xscrollbar -side bottom -fill x    -expand 0
    pack .hv.text       -side left   -fill both -expand 1
    pack .hv.menu1 .hv.file .hv.entry .hv.status .hv.menu2 .hv.help .hv.quit \
         -in .hv.frame1 -side left
    pack .hv.reverse .hv.forward .hv.histbtn .hv.msg \
         -in .hv.frame2 -side left

    .hv.forward configure -state disabled
    .hv.reverse configure -state disabled

    # set up some sample keyboard bindings for the text widget
    bind .hv.entry <Return> {render $hv_Url}
    bind all <End>   {.hv.text yview end}
    bind all <Home>  {.hv.text yview 0.0}
    bind all <Next>  {.hv.text yview scroll 1 page}
    bind all <Prior> {.hv.text yview scroll -1 page}

    # I'm constantly being criticized for never using menus.
    # so here's a menu.  So there.

    menu .hv.menu1.f -tearoff no
    .hv.menu1.f add command -label "$m(open)..." -command LoadFile -background skyblue
    .hv.menu1.f add separator
    .hv.menu1.f add command -label "$m(close)" -command {destroy .hv} -background skyblue

    menu .hv.menu2.m
    .hv.menu2.m add command -label "$m(option_menu)"
    .hv.menu2.m add separator
    .hv.menu2.m add command -label "$m(font_size)" -foreground red 
    .hv.menu2.m add radiobutton -label "$m(small)" -value 0   -variable hv_Size \
		 -command {HMset_state .hv.text -size $hv_Size; render $hv_Url}
    .hv.menu2.m add radiobutton -label "$m(medium)" -value 4  -variable hv_Size \
		 -command {HMset_state .hv.text -size $hv_Size; render $hv_Url}
    .hv.menu2.m add radiobutton -label "$m(large)" -value 12  -variable hv_Size \
		 -command {HMset_state .hv.text -size $hv_Size; render $hv_Url}
    .hv.menu2.m add separator
    .hv.menu2.m add command -label "$m(indent_level)" -foreground red
    .hv.menu2.m add radiobutton -label  "$m(small)" -value 0.6 -variable hv_Indent \
		 -command {HMset_indent .hv.text $hv_Indent}
    .hv.menu2.m add radiobutton -label "$m(medium)"  -value 1.2 -variable hv_Indent \
		 -command {HMset_indent .hv.text $hv_Indent}
    .hv.menu2.m add radiobutton -label  "$m(large)" -value 2.4 -variable hv_Indent \
		 -command {HMset_indent .hv.text $hv_Indent}
}
#-----------------------------------------------------------
# URLɽ椫/Υڡɽ
#
# 2002 Aug 26 by R.Kawagishi
#----------------------------------------------------------- 
proc reverse_url {} {
    global fax m
    global Url_history url_hist_ptr url_hist_flag

    if {$url_hist_ptr > 0} then {
        set url_hist_ptr [expr $url_hist_ptr -1]
        if {$url_hist_ptr == 0} then {
            .hv.forward configure -state disabled
        }
        set url_hist_flag 1
        render [lindex $Url_history $url_hist_ptr]
    }
}

proc forward_url {} {
    global fax m
    global Url_history url_hist_ptr url_hist_flag

    set ptr_limit [expr [llength $Url_history] -1]
    if {$url_hist_ptr < $ptr_limit } then {
        set url_hist_ptr [expr $url_hist_ptr +1]
        if {$url_hist_ptr == $ptr_limit} then {
            .hv.forward configure -state disabled
        }
        set url_hist_flag 1
        render [lindex $Url_history $url_hist_ptr]
    }
}

#-----------------------------------------------------------
# إɽɽ
#
# 2002 Aug 26 by R.Kawagishi
#----------------------------------------------------------- 
proc help_history {} {
    global fax m
    global Url_history hist_item

    toplevel .hv.hist
    wm title .hv.hist "TkScanFax Help Page History"

    entry     .hv.hist.ent -textvariable hist_item -width 70 -state disabled \
                           -bg cornsilk
    listbox   .hv.hist.list -width 70 -height 10 -background white \
              -selectbackground skyblue \
              -yscrollcommand ".hv.hist.yscr set" \
              -xscrollcommand ".hv.hist.xscr set"
    scrollbar .hv.hist.yscr -command ".hv.hist.list yview"
    scrollbar .hv.hist.xscr -command ".hv.hist.list xview" -orient horizontal
    mscroll   .hv.hist.list 
    button    .hv.hist.ok -text "$m(ok)" -command {
              set page [.hv.hist.ent get]
              destroy .hv.hist
              set url_hist_flag 1
              render $page
    }
    button .hv.hist.delete -text "$m(delete_from_history)" -command {
           set index [.hv.hist.list curselection]
	   .hv.hist.list delete $index
	   set Url_history [lreplace $Url_history $index $index]
    }
    button .hv.hist.cancel -text "$m(cancel)"   -command {destroy .hv.hist}

    grid .hv.hist.ent    -column 1  -columnspan 5 -row 1
    grid .hv.hist.list   -column 1  -columnspan 5 -row 2
    grid .hv.hist.yscr   -column 6  -columnspan 1 -row 2 -sticky ns
    grid .hv.hist.xscr   -column 1  -columnspan 5 -row 3 -sticky we
    grid .hv.hist.ok     -column 2  -columnspan 1 -row 4 -sticky we
    grid .hv.hist.delete -column 3  -columnspan 1 -row 4 -sticky we
    grid .hv.hist.cancel -column 4  -columnspan 1 -row 4 -sticky we

    set item_num [llength $Url_history]
    for {set i 0} {$i < $item_num} {incr i} {
        .hv.hist.list insert end [lindex $Url_history $i]
    }
    .hv.hist.list selection set 0
    set hist_item [.hv.hist.list get 0]

    bind .hv.hist.list <ButtonRelease-1> {
         set hist_item [.hv.hist.list get @%x,%y]
    }
    bind .hv.hist.list <Double-1> {
         set hist_item [.hv.hist.list get @%x,%y]
         set page [.hv.hist.ent get]
         destroy .hv.hist
         set url_hist_flag 1
         render $page
    }
}
#-----------------------------------------------------------
# 2002 Aug 26 by R.Kawagishi
#-----------------------------------------------------------
# 11 Oct 2002
proc hv_help {} {
    global fax m
    return [fax_help "HTML Viewer" $m(hv_help)]
}

#-----------------------------------------------------------
# Go render a page.  We have to make sure we don't render one page while
# still rendering the previous one.  If we get here from a recursive 
# invocation of the event loop, cancel whatever we were rendering when
# we were called.
# If we have a fragment name, try to go there.
#-----------------------------------------------------------
# 2002 Aug 26 by R.Kawagishi add reverse/forward button control

proc render {file} {
    global fax m
    global HM.text
    global hv_Url hv_Running hv_message
    global Url_history url_hist_ptr url_hist_flag

    .hv.reverse configure -state disabled
    .hv.forward configure -state disabled
    .hv.histbtn configure -state disabled

	set fragment ""
	regexp {([^#]*)#(.+)} $file dummy file fragment
	if {$file == "" && $fragment != ""} {
		HMgoto .hv.text $fragment
		return
	}
	HMreset_win .hv.text
	set hv_Running busy
	set hv_message "Displaying $file"
	update idletasks
	if {$fragment != ""} {
		HMgoto .hv.text $fragment
	}
	set hv_Url $file
	HMparse_html [get_html $file] {HMrender .hv.text}
	set hv_Running ready
	HMset_state .hv.text -stop 1	;# stop rendering previous page if busy
	set hv_message ""

    if {$url_hist_flag == 0} then {
        lappend Url_history $file
        set url_hist_ptr [expr [llength $Url_history] -1]
    }
    set url_hist_flag 0

    if {$url_hist_ptr == 0} then {
        .hv.reverse configure -state disabled
    } else {
        .hv.reverse configure -state normal
    }

    set ptr_limit [expr [llength $Url_history] -1]
    if {$url_hist_ptr < $ptr_limit} then {
        .hv.forward configure -state normal
    } else {
        .hv.forward configure -state disabled
    }
    .hv.histbtn configure -state normal

#tkwait visibility .hv.reverse
#tkwait visibility .hv.forward
#puts "url_hist_ptr = $url_hist_ptr"
#puts "ptr_limit = $ptr_limit"
#puts "Url_history = $Url_history"
#puts " "
}

#-----------------------------------------------------------
# given a file name, return its html, or invent some html if the file can't
# be opened.
#-----------------------------------------------------------
proc get_html {file} {
	global hv_Home
	if {[catch {set fd [open $file]} msg]} {
		return "
			<title>Bad file $file</title>
			<h1>Error reading $file</h1><p>
			$msg<hr>
			<a href=$hv_Home>Go home</a>
		"
	}
	# special for tkscanfax to set encoding
	set d [file split $file]
	set d [lreplace $d end end]
	set d [lindex $d end]
	if { $d == {ja} || $d == {ja_JP.eucJP} } {
		catch { fconfigure $fd -encoding euc-jp }
	}

	set result [read $fd]
	close $fd
	return $result
}

#-----------------------------------------------------------
# Override the library link-callback routine for the sample app.
# It only handles the simple cases.
#-----------------------------------------------------------
proc HMlink_callback {win href} {
	global hv_Url
	if {[string match #* $href]} {
		render $href
		return
	}
	if {[string match /* $href]} {
		set hv_Url $href
	} else {
		set hv_Url [file dirname $hv_Url]/$href
	}
	update
	render $hv_Url
}

#-----------------------------------------------------------
# Supply an image callback function
# Read in an image if we don't already have one
# callback to library for display
#-----------------------------------------------------------
proc HMset_image {win handle src} {
	global hv_Url hv_message
	if {[string match /* $src]} {
		set image $src
	} else {
		set image [file dirname $hv_Url]/$src
	}
	set hv_message "fetching image $image"
	update
	if {[string first " $image " " [image names] "] >= 0} {
		HMgot_image $handle $image
	} else {
		set type photo
		if {[file extension $image] == ".bmp"} {set type bitmap}
		catch {image create $type $image -file $image} image
		HMgot_image $handle $image
	}
}

#-----------------------------------------------------------
# Handle base tags.  This breaks if more than 1 base tag is in the document
#-----------------------------------------------------------
proc HMtag_base {win param text} {
	global hv_Url
	upvar #0 HM$win var
	HMextract_param $param href hv_Url
}

#-----------------------------------------------------------
# downloading fonts can take a long time.  We'll override the default
# font-setting routine to permit better user feedback on fonts.  We'll
# keep our own list of installed fonts on the side, to guess when delays
# are likely

proc HMset_font {win tag font} {
    global fax m
    global hv_message Fonts

	if {![info exists Fonts($font)]} {
		set Fonts($font) 1
		.hv.msg configure -fg blue
		set hv_message "downloading font $font"
		update
	}
	.hv.msg configure -fg black
	set hv_message ""
	catch {$win tag configure $tag -font $font} hv_message
}

#-----------------------------------------------------------
# Lets invent a new HTML tag, just for fun.
# Change the color of the text. Use html tags of the form:
# <color value=blue> ... </color>
# We can invent a new tag for the display stack.  If it starts with "T"
# it will automatically get mapped directly to a text widget tag.
#-----------------------------------------------------------
proc HMtag_color {win param text} {
	upvar #0 HM$win var
	set value bad_color
	HMextract_param $param value
	$win tag configure $value -foreground $value
	HMstack $win "" "Tcolor $value"
}

proc HMtag_/color {win param text} {
	upvar #0 HM$win var
	HMstack $win / "Tcolor {}"
}

#-----------------------------------------------------------
# Add a font size manipulation primitive, so we can use this sample program
# for on-line presentations.  sizes prefixed with + or - are relative.
#  <font size=[+-]3>  ..... </font>.  Note that this is not the same as
# Netscape's <font> tag.
#-----------------------------------------------------------
proc HMtag_font {win param text} {
	upvar #0 HM$win var
	set size 0; set sign ""
	HMextract_param $param size
	regexp {([+-])? *([0-9]+)} $size dummy sign size
	if {$sign != ""} {
		set size [expr [lindex $var(size) end] $sign $size]
	}
	HMstack $win {} "size $size"
}

#-----------------------------------------------------------
# This version is closer to what Netscape does
#-----------------------------------------------------------
proc HMtag_font {win param text} {
	upvar #0 HM$win var
	set size 0; set sign ""
	HMextract_param $param size
	regexp {([+-])? *([0-9]+)} $size dummy sign size
	if {$sign != ""} {
		set size [expr [lindex $var(size) end] $sign  $size*2]
		HMstack $win {} "size $size"
	} else {
		HMstack $win {} "size [expr 10 + 2 * $size]"
	}
}

proc HMtag_/font {win param text} {
	upvar #0 HM$win var
	HMstack $win / "size {}"
}
